from si_tests.utils import packaging_version as version
from si_tests.utils import utils

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from si_tests.managers.kaas_manager import Cluster


class WorkaroundCheckManagerBase(object):
    """Common methods useful for workarounds check"""

    def __init__(self, cluster: "Cluster"):
        self._cluster: "Cluster" = cluster

    @property
    def cluster(self) -> "Cluster":
        return self._cluster

    @property
    def k8sclient(self):
        return self._cluster.k8sclient

    @property
    def mcc_version(self):
        """Active MCC release version"""
        mgmt_cluster = self._cluster._manager.get_mgmt_cluster()
        return mgmt_cluster.get_kaasrelease_version()

    @property
    def clusterrelease_version(self):
        """Current Cluster version"""
        return self._cluster.clusterrelease_version


class WorkaroundCheckManager(WorkaroundCheckManagerBase):
    """Workaround check manager

    Purpose of this class is to keep all the checks for workarounds in a single place.

    - Each method should return 'True' if the related workaround should be applied.
      Please don't add any waiters/actions here. Only checks for versions/providers/cluster objects/etc...

    - Methods should be named using PRODX bugs/tasks, for example:

          def prodx_12345(self):

    - Please clear the code from outdated workarounds after the related tasks are resolved
    """

    def prodx_34281(self):
        # https://mirantis.jira.com/browse/PRODX-34281 , fix version: 2.24.3
        if version.parse(self.mcc_version) < version.parse("kaas-2-24-3") and self.cluster.provider in [
                utils.Provider.baremetal, utils.Provider.equinixmetalv2, utils.Provider.vsphere]:
            # apply workaround for PRODX-34281 if MCC release is lower than 2.24.3
            return True

        return False
