import pytest

from si_tests import logger
from si_tests import settings
from si_tests.utils import packaging_version as version
from si_tests.managers.kaas_manager import Manager

LOG = logger.logger


class TestCacheWarmup():
    def test_cache_warmup(self, kaas_manager: Manager, show_step):
        """Warmup regional cache before child cluster(s) deployment

         Scenario:
             1. Create Cache Warmup request
             2. Wait for job start
             3. Verify warmup regional status for cluster
             4. Verify cluster warmup condition
         """
        # TARGET_CLUSTER has to be either management or regional
        cluster_name = settings.TARGET_CLUSTER
        ns = kaas_manager.get_namespace(namespace='default')
        cluster = ns.get_cluster(cluster_name)
        mke_creleases = kaas_manager.si_config.version_generators.gen_child_cluster_release_versions('mke')
        mosk_creleases = kaas_manager.si_config.version_generators.gen_child_cluster_release_versions('mosk')
        mke_supported_warmup = [release for release in mke_creleases
                                if version.parse(release) > version.parse('mke-11-7-4-3-5-7')]
        mosk_supported_warmup = [release for release in mosk_creleases
                                 if version.parse(release) > version.parse('mosk-12-7-4-23-1-4')]
        releases_for_warm_up = mke_supported_warmup + mosk_supported_warmup

        LOG.info(f"Generated releases to warm-up {releases_for_warm_up}")

        cluster_releases = settings.KAAS_CACHE_WARMUP_CLUSTER_RELEASES.split(',') if \
            settings.KAAS_CACHE_WARMUP_CLUSTER_RELEASES else releases_for_warm_up

        openstack_releases = settings.KAAS_CACHE_WARMUP_OPENSTACK_RELEASES.split(',') if \
            settings.KAAS_CACHE_WARMUP_OPENSTACK_RELEASES else []

        if not cluster_releases and not openstack_releases:
            pytest.skip("KAAS_CACHE_WARMUP_OPENSTACK_RELEASES "
                        "and KAAS_CACHE_WARMUP_CLUSTER_RELEASES are empty"
                        "so nothing to warm up")
        show_step(1)
        cwr = cluster.create_cache_warmup_request(cluster_name, cluster_releases, openstack_releases,
                                                  settings.KAAS_CACHE_WARMUP_OPENSTACK_ONLY)

        show_step(2)
        # Mgmt or Regional cluster resides in default namespace only
        cluster.wait_warmup_job_creation(namespace='kaas')

        show_step(3)
        cluster.wait_for_cluster_warmup_success(cwr.creation_timestamp,
                                                timeout=settings.KAAS_CACHE_WARMUP_READINESS_TIMEOUT)

        show_step(4)
        cluster.wait_for_cluster_condition('CacheWarmup')
